import sys
import os

import pytest

from amarium.encryption import generate_asymmetric_keys, encrypt_file, decrypt_file


def test_asymmetric_key_generation():
    # case 1
    key_size = 2048
    public_key_name = "tests/public_test.pem"
    private_key_name = "tests/private_test.pem"
    if os.path.isfile(public_key_name):
        os.remove(public_key_name)

    if os.path.isfile(private_key_name):
        os.remove(private_key_name)

    generate_asymmetric_keys(
        key_size=key_size,
        private_key_name=private_key_name,
        public_key_name=public_key_name,
        write_secret=False,
    )
    assert "public_test.pem" in os.listdir("tests")
    assert "private_test.pem" not in os.listdir("tests")
    # case 2
    generate_asymmetric_keys(
        key_size=key_size,
        private_key_name=private_key_name,
        public_key_name=public_key_name,
        write_secret=True,
    )
    assert "public_test.pem" in os.listdir("tests")
    assert "private_test.pem" in os.listdir("tests")


def test_file_encryption():
    message = "Amarium is a decent library to write and read files"

    file_name = "test_encryption_files/test_encrypt.txt"
    if os.path.isfile(file_name):
        os.remove(file_name)
    with open(file_name, "w") as test_file:
        test_file.write(message)

    save_file = "test_encryption_files/test_encrypted_test_file.txt"
    public_key_file = "tests/public_test.pem"

    in_place = False

    encrypt_file(
        file_name=file_name,
        public_key_file=public_key_file,
        save_file=save_file,
        in_place=in_place,
    )
    assert os.path.isfile(save_file)

    in_place = True
    encrypt_file(
        file_name=file_name, public_key_file=public_key_file, in_place=in_place
    )


def test_decryption():
    encrypted_file = "test_encryption_files/test_encrypted_test_file.txt"
    private_key_path = "tests/private_test.pem"

    message = decrypt_file(
        encrypted_file=encrypted_file, private_key_path=private_key_path
    )
    assert message == b"Amarium is a decent library to write and read files"



from _typeshed import Incomplete
from typing import Any, Dict, List, Optional, Tuple

LOGGER: Incomplete

def prepare_file_name_saving(
    file_name: str,
    prefix: Optional[str | None] = None,
    suffix: Optional[str | None] = None,
) -> str: ...
def insert_string_in_file_name(file_name: str, insertion: str, suffix: str) -> str: ...
def make_full_filename(prefix: str, file_name: str) -> str: ...
def load_json_from_file(file_name: str) -> Dict[Any, Any]: ...
def save_json_to_file(
    dictionary: Dict[Any, Any], file_name: Optional[str] = ..., suffix: str = ...
) -> None: ...
def make_date_file_name(
    prefix: str = ..., file_name: str = ..., suffix: str = ...
) -> str: ...
def delete_file(file_name: str) -> None: ...
def search_subdirs(dir_name: str) -> Tuple[List[str], List[str]]: ...
def write_tar(target_dir: str, file_name: str) -> None: ...
def copy_dir(old_dir: str, new_dir: str, overwrite: bool = ...) -> None: ...



"""Module to provide sanity checks for user inputs and other routine quality
checks during execution.

:author: Julian M. Kleber
"""

import os
from collections.abc import Iterable
from typing import List, Any
import shutil


import logging

LOGGER = logging.getLogger(__name__)


def check_header_and_data(data: List[Any], header: List[Any]) -> None:
    """The check_header_and_data function checks that the length of the data
    and header lists are equal. If they are not, it raises a RuntimeError with
    an appropriate message.

    :param data: List[Any]: Used to Pass in the data to be written.
    :param header: List[Any]: Used to Ensure that the header and data
        have the same number of fields.
    :return: A boolean value.  :doc-author: Julian M. Kleber
    """

    if len(data) == len(header):
        pass
    else:
        raise RuntimeError(
            "Input data and input header do not have the "
            "same number of fields. "
            "Please ensure an appropriate data structure."
        )


def check_make_file_name_suffix(file_name: str, suffix: str = "") -> str:
    """The check_name_plot function checks that the file_name ends with .png.
    If it does not, then check_name_plot appends .png to the end of the
    file_name.

    :param file_name: Used to Specify the name of the file to be
        plotted.
    :return: The file_name with the correct file extension. :doc-
        author: Trelent
    """
    name = file_name
    ext = os.path.splitext(file_name)[1]
    if not ext:
        if not suffix.startswith("."):
            suffix = "." + suffix
        name += suffix
    return name


def check_make_subdirs(dir_names: Iterable[str]) -> str:
    from .utils import attach_slash

    final_dir = ""
    for directory in dir_names:
        final_dir += attach_slash(directory)

    check_make_dir(final_dir)
    logging.info("Created dir " + final_dir)
    return final_dir


def check_delete_dir(dir_name: str) -> None:
    """The check_delete_dir function checks if a directory exists and deletes
    it.

    :param dir_name: str: Used to Specify the directory name.
    :return: None. :doc-author: Julian M. Kleber
    """

    if os.path.isdir(dir_name):
        shutil.rmtree(dir_name)


def check_make_dir(dir_name: str) -> None:
    """The check_make_dir function checks if a directory exists. If it does not
    exist, the function creates it.

    :param dir_name: str: Used to Specify the folder name.
    :return: None.  :doc-author: Trelent
    """

    if dir_name == "":
        return None

    check_folder = os.path.isdir(dir_name)
    logging.info("Checked the directory %s", dir_name)

    if not check_folder:
        os.makedirs(dir_name)
        logging.info("Created folder : %s", dir_name)

    else:
        logging.info("%s folder already exists.", dir_name)



from typing import Any, List

def check_header_and_data(data: List[Any], header: List[Any]) -> None: ...
def check_make_file_name_suffix(file_name: str, suffix: str = ...) -> str: ...
def check_make_dir(dir_name: str) -> None: ...



"""Module for encryption of files. Useful for all kinds of situtations.

:author: Julian M. Kleber
"""

from typing import Tuple, Optional

from cryptography.hazmat.backends import default_backend
from cryptography.hazmat.primitives.asymmetric import rsa, padding
from cryptography.hazmat.primitives import serialization, hashes

from .utils import prepare_file_name_saving


def decrypt_file(encrypted_file: str, private_key_path: str):
    """The decrypt_file function takes in an encrypted file and a private key,
    then decrypts the file using the private key.

    :param encrypted_file: str: Used to Pass the file name of the
        encrypted file to be decrypted.
    :param private_key: str: Used to Read the private key file and
        decrypt the encrypted message.
    :return: The original message.  :doc-author: Julian M. Kleber
    """

    with open(encrypted_file, "rb") as ecf:
        encrypted = (
            ecf.read()
        )  # From before (could have been stored then read back here)

    with open(private_key_path, "rb") as key_file:
        private_key = serialization.load_pem_private_key(key_file.read(), password=None)

    original_message = private_key.decrypt(
        encrypted,
        padding.OAEP(
            mgf=padding.MGF1(algorithm=hashes.SHA512()),
            algorithm=hashes.SHA512(),
            label=None,
        ),
    )
    return original_message


def encrypt_file(
    file_name: str,
    public_key_file: str,
    in_place: bool = False,
    save_file: Optional[str] = None,
) -> None:
    """The encrypt_file function takes a file name, public key, and optional
    in-place flag. It then encrypts the contents of the file using the provided
    public key. If in_place is True, it will overwrite the original file with
    its encrypted version. Otherwise it will create a new encrypted version of
    that file.

    :param file_name:str: Used to Specify the name of the file to be
    encrypted. :param public_key:str: Used to Specify the name of the
    public key file. :param in_place:bool=False: Used to Determine
    whether the file should be overwritten or not. :param save_
    file:Optional[str]=None:
     :param file_name: str: Used to Specify the name of the file to be
    encrypted. :param public_key: str: Used to Specify the name of the
    public key     file. :param in_place: bool=False: Used to Determine
    whether the file     should be overwritten or not. :param save_
    file:Optional[str]=None:
    :param file_name: str: Used to Specify the name of the file to be
        encrypted.
    :param public_key: str: Used to Specify the name of the public key
        file.
    :param in_place: bool=False: Used to Determine whether the file
        should be overwritten or not. :param save_
        file:Optional[str]=None:
    :param file_name: str: Used to Specify the name of the file to be
        encrypted.
    :param public_key: str: Used to Specify the name of the public key
        file.
    :param in_place: bool=False: Used to Determine whether the file
        should be overwritten or not. :param save_
        file:Optional[str]=None:
    :param file_name: str: Used to Specify the name of the file to be
        encrypted.
    :param public_key: str: Used to Specify the name of the public key
        file.
    :param in_place: bool=False: Used to Determine whether the file
        should be overwritten or not.
    :param save_file: Optional[str]=None: Used to Allow the user to
        specify a file name for saving the encrypted file.
    :return: None.  :doc-author: Julian M. Kleber
    """

    with open(public_key_file, "rb") as key_file:
        public_key = serialization.load_pem_public_key(
            key_file.read(), backend=default_backend()
        )

    if in_place:
        save_file = file_name

    with open(file_name, "r", encoding="utf-8") as to_encrypt:
        file_content = str.encode(to_encrypt.read())

    encrypted = public_key.encrypt(
        file_content,
        padding.OAEP(
            mgf=padding.MGF1(algorithm=hashes.SHA512()),
            algorithm=hashes.SHA512(),
            label=None,
        ),
    )

    with open(save_file, "wb") as encrypted_file:
        encrypted_file.write(encrypted)


def generate_asymmetric_keys(
    key_size: int, private_key_name: str, public_key_name: str, write_secret: bool
) -> Tuple[bytes, bytes]:
    """The generate_asymmetric_keys function generates a private key and public
    key pair. The keys are generated using the RSA algorithm with a specified
    bit size. The keys are then written to files in PEM format.

    :param key_size: int: Used to specify the size of the key that will
        be generated.
    :param private_key_name: str: Used to Name the private key file.
    :param public_key_name: str: Used to specify the name of the public
        key file.
    :return: The private and public keys.  :doc-author: Julian M. Kleber
    """

    private_key_name = prepare_file_name_saving(private_key_name)
    public_key_name = prepare_file_name_saving(public_key_name)
    private_key = rsa.generate_private_key(
        public_exponent=65537, key_size=key_size, backend=default_backend()
    )

    secret_pem = private_key.private_bytes(
        encoding=serialization.Encoding.PEM,
        format=serialization.PrivateFormat.PKCS8,
        encryption_algorithm=serialization.NoEncryption(),
    )

    if write_secret:
        with open(
            private_key_name, "wb"
        ) as key_file:  # writes in binary mode if there a special file errors
            key_file.write(secret_pem)

    public_key = private_key.public_key()

    public_pem = public_key.public_bytes(
        encoding=serialization.Encoding.PEM,
        format=serialization.PublicFormat.SubjectPublicKeyInfo,
    )

    with open(public_key_name, "wb") as key_file:
        key_file.write(public_pem)

    return public_key, private_key



from .utils import prepare_file_name_saving as prepare_file_name_saving
from typing import Tuple

def generate_asymmetric_keys(
    key_size: int, private_key_name: str, public_key_name: str
) -> Tuple[bytes, bytes]: ...
